package de.hs_mannheim.informatik.domain;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import java.io.*;

public class Highscores {
    private static final Logger logger = LogManager.getLogger(Highscores.class);

    public static void newRecord(String resourcePath, String username, String time) throws IOException {
        String timePart = time.substring("Zeit: ".length()).trim();
        String[] parts = resourcePath.split("/");
        String jarFileName = parts[parts.length - 1]; 
        String newName = jarFileName.replace(".csv", ".txt");

        File localFolder = new File("resources/Hitori_Highscores");
        if (!localFolder.exists() && !localFolder.mkdirs()) {
            logger.error("Failed to create output folder: {}", localFolder.getAbsolutePath());
            throw new IOException("Failed to create output folder: " + localFolder);
        }

        File localFile = new File(localFolder, newName);

        try (InputStream inputStream = Highscores.class.getResourceAsStream(resourcePath)) {
            if (inputStream == null) {
                logger.warn("Resource not found in JAR: {}", resourcePath);
            } else {
                try (BufferedReader br = new BufferedReader(new InputStreamReader(inputStream))) {
                    String line;
                    while ((line = br.readLine()) != null) {
                        logger.debug("Existing JAR record: {}", line);
                    }
                }
            }
        }

        String entry = timePart + " " + username;
        logger.info("Adding new record: {} to file: {}", entry, localFile.getAbsolutePath());
        try (BufferedWriter writer = new BufferedWriter(new FileWriter(localFile, true))) {
            writer.write(entry);
            writer.newLine();
        } catch (IOException e) {
            logger.error("Error handling local highscore file: {}", localFile.getAbsolutePath(), e);
            throw e;
        }
    }
}
