package de.hs_mannheim.informatik.gui;

import java.awt.*;
import java.util.Stack;
import java.util.Timer;
import java.util.TimerTask; 
import java.io.*;
import javax.swing.*;

import de.hs_mannheim.informatik.facade.HitoriAPI;

public class GameGUI extends JFrame {
	
	private Timer timer = new Timer();
	private long startTime;
	private long elapsedSeconds;
    private byte[] gameData;
    
    public Object[] giveFrame(CardLayout cl, JPanel main){
    	Object[] frameArray = new Object[2];
    	frameArray[0] = cl;
    	frameArray[1] = main;
    	return frameArray;
    }
    
    public void paintGame(CardLayout cl, JPanel main, InputStream inputStream, Stack<String> madeMoves, String[][] data, String path0, String[][] colors) throws IOException{
    	elapsedSeconds = 0;
        gameData = inputStream.readAllBytes();
        JButton[][] buttons = makeButtonArray(data);
        String[] filepath = new String[2];
        int num = buttons.length;
        JPanel gameGrid = new JPanel(new GridLayout(num,num,0,0));
        for(int i=0;i<num;i++){
            for(int j=0;j<num;j++){
                JButton button = buttons[i][j];
                Color foreground = Color.BLACK;
                Color background = Color.WHITE;
                changeButtonColor(button, foreground, background);
                gameGrid.add(button);
                String[] pos = HitoriAPI.getCordsCatcher(i,j);
                button.addActionListener(e -> {paintButton(cl, main, button, pos, colors , madeMoves, gameGrid, buttons);});
            }
        }
        JPanel mainPanel = new JPanel(new BorderLayout());
        mainPanel.add(gameGrid, BorderLayout.CENTER);
        JPanel buttonGrid = new JPanel(new GridLayout(1,4,10,10));
        JButton aufgebenButton = new JButton("Aufgeben");
        aufgebenButton.addActionListener(e -> {
            filepath[0] = "";
            filepath[1] = "";      
            cl.show(main, "HAUPT");
            });
        JButton zurückButton = new JButton("Zurück");
        zurückButton.addActionListener(e -> {backOneStep(cl, main, madeMoves, buttons, colors, gameGrid);});
        JButton resetButton = new JButton("Zurücksetzen");
        resetButton.addActionListener(e -> {try {
        	totalResetButton(inputStream, cl, main, buttons, colors, madeMoves, data, path0);
        } catch (IOException e1) {
            e1.printStackTrace();
        }});
        JButton abgebenButton = new JButton("Abgeben");
        boolean[] levelFinished = {false};
        JPanel topGrid = new JPanel(new GridLayout(1,2,10,10));
        mainPanel.add(topGrid, BorderLayout.NORTH);
        JLabel timeLabel = new JLabel("Zeit: 00:00");
        topGrid.add(timeLabel);
        topGrid.revalidate();
        topGrid.repaint();
        boolean isOkay = new HitoriAPI().abgabeMöglichCatcher(inputStream, data, colors);
        levelFinished[0] = isOkay; 
        Timer timer = new Timer();
        startTimer();
        timer.scheduleAtFixedRate(new TimerTask() {
            @Override
            public void run() {
                elapsedSeconds = (System.currentTimeMillis() - startTime) / 1000;
                long minutes = elapsedSeconds % 3600 / 60;
                long seconds = elapsedSeconds % 60;
                String newTime = String.format("Zeit: %02d:%02d", minutes, seconds);

                SwingUtilities.invokeLater(() -> {
                    timeLabel.setText(newTime);
                    timeLabel.revalidate();
                    timeLabel.repaint();
                });
            }
        }, 0, 1000);
        JLabel platzhalter = new JLabel("");
        topGrid.add(platzhalter);
        abgebenButton.addActionListener(e -> {
            try {
            	InputStream newStream = new ByteArrayInputStream(gameData);
            	levelFinished[0] = new HitoriAPI().abgabeMöglichCatcher(newStream, data, colors); 
            } catch (FileNotFoundException e1) {
                e1.printStackTrace();
            }
            if (levelFinished[0] == true) {
                String endtime = stopTimer();
                timer.cancel();
                try {
                    NewEntryGUI.finish(cl, main, endtime, filepath, path0);
                } catch (FileNotFoundException e1) {
                    e1.printStackTrace();
                }
            } else {
            	platzhalter.setText("Abgabe nicht richtig!");
                mainPanel.revalidate();
                mainPanel.repaint();
            }
        });
        buttonGrid.add(aufgebenButton);
        buttonGrid.add(zurückButton);
        buttonGrid.add(resetButton);
        buttonGrid.add(abgebenButton);
        mainPanel.add(buttonGrid, BorderLayout.SOUTH);
        mainPanel.setVisible(true);
        mainPanel.setSize(700,700);
        main.add(mainPanel, "GAME");
        cl.show(main, "GAME");
    }

    private void changeButtonColor(JButton button, Color foreground, Color background) {
    	button.setOpaque(true);
        button.setForeground(foreground);
    	button.setContentAreaFilled(true);
     	button.setBorderPainted(false);
     	button.setFocusPainted(false);
     	button.setBackground(background);
    }
    
    public void paintButton(CardLayout cl, JPanel main, JButton button, String[] pos, String[][] colors, Stack<String> madeMoves, JPanel grid, JButton[][] buttons){
        int i = Integer.parseInt(pos[0]);
        int j = Integer.parseInt(pos[1]);
        String col = colors[i][j];
        String lastLetterString = String.valueOf(col.charAt(col.length()-1));
        Color foreground;
        Color background;
    	switch(lastLetterString) {
	    	case "W":
	    		foreground = Color.BLACK;
	    		background = Color.lightGray;
	    		lastLetterString = "G";
	    		changeButtonColor(button, foreground, background);
	    		break;
	    	case "G":
	    		foreground = Color.WHITE;
	    		background = Color.BLACK;
	    		lastLetterString = "B";
	    		changeButtonColor(button, foreground, background);
	    		break;
	    	case "B":
	    		foreground = Color.BLACK;
	    		background = Color.WHITE;
	    		lastLetterString = "W";
	    		changeButtonColor(button, foreground, background);
	    		break;
    	}
    	HitoriAPI.addMoveCatcher(colors, i,j,lastLetterString,madeMoves);
    }
    
    public void backOneStep(CardLayout cl, JPanel main, Stack<String> movesMade, JButton[][] buttons, String[][] colors, JPanel grid){
    	String move = movesMade.pop();
	    String[] line = move.split("\\.");
        int i = Integer.parseInt(line[0]); 
        int j = Integer.parseInt(line[1]); 
        String color = line[2];
        JButton button = buttons[i][j];
        Color foreground;
        Color background;
        switch(color) {
        	case "W":
            	foreground = Color.WHITE;
        		background = Color.BLACK;
            	changeButtonColor(button, foreground, background);
            	break;
            case "G":
            	foreground = Color.BLACK;
        		background = Color.WHITE;
            	changeButtonColor(button, foreground, background);
            	break;
            case "B":
            	foreground = Color.BLACK;
        		background = Color.lightGray;
            	changeButtonColor(button, foreground, background);
            	break;   	
        }
        removeMove(colors, i, j, grid, buttons);
    }
    
    private void removeMove(String[][] colors, int i, int j, JPanel grid, JButton[][] buttons) {
    	String colorString = colors[i][j]; 
        String colorStringWithoutLastElement = colorString.substring(0, colorString.length() - 1);
        colors[i][j] = colorStringWithoutLastElement;
        grid.repaint();
        gridUpdate(grid, buttons);
    }
       
	public void gridUpdate(JPanel grid, JButton[][] buttons){
        grid.removeAll();
        grid.repaint();
        for(int i = 0; i<buttons.length; i++){
            for(int j = 0; j<buttons.length;j++){
                JButton button = buttons[i][j];
                grid.add(button);
                grid.repaint();
            }
        }
    }
    
    public String startTimer() {
        startTime = System.currentTimeMillis();
        elapsedSeconds = 0; 
        return "Zeit: 00:00";
    }
    
    public String stopTimer() {
        timer.cancel();
        long minutes = elapsedSeconds % 3600 / 60;
        long seconds = elapsedSeconds % 60;
        String endtime = String.format("Zeit: %02d:%02d", minutes, seconds);
        elapsedSeconds = 0; 
        return endtime;
    }

    public JButton[][] makeButtonArray(String[][] data){
        JButton[][] buttons = new JButton[data.length][data.length];
        for(int i=0;i<data.length;i++){
            for(int j=0;j<data.length;j++){
                String number = data[i][j];
                JButton button = new JButton(number);
                buttons[i][j] = button;
            }
        }
        return buttons;
    }
    
    public void totalResetButton(InputStream inputStream, CardLayout cl, JPanel main, JButton[][] buttonArray, String[][] colors,Stack<String> madeMoves,String[][] data,String path) throws IOException{
    	madeMoves.clear();
            for(int i = 0; i<data.length;i++){
                for(int j = 0; j<data.length;j++){
                    colors[i][j] = "W";
                    buttonArray[i][j] = null;
            }
        paintGame(cl, main, inputStream, madeMoves, data, path, colors);
        }
    }
}
