package de.hs_mannheim.informatik.test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertNotEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.io.FileNotFoundException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;

import org.junit.jupiter.api.Test;

import de.hs_mannheim.informatik.domain.CSVReader;

public class CSVReaderTest {
    @Test
    void getSolutionTest1() throws FileNotFoundException {
    	String[] ergebnis4x4 = {"0,1", "1,3", "2,1", "3,0", "3,3"};
    	String path = "/Hitori_Spielfelder/Hitori4x4_leicht.csv";
    	InputStream inputStream = getClass().getResourceAsStream(path);
    	ArrayList<String> filteredData = new CSVReader().getSolution(inputStream);
    	assertEquals(5, filteredData.size());
    	assertEquals(ergebnis4x4[0], filteredData.get(0));
    	assertEquals(ergebnis4x4[1], filteredData.get(1));
    	assertEquals(ergebnis4x4[2], filteredData.get(2));
    	assertEquals(ergebnis4x4[3], filteredData.get(3));
    	assertEquals(ergebnis4x4[4], filteredData.get(4));
    }
    
    @Test
    void getSolutionTest2() throws FileNotFoundException {
    	String[] ergebnis4x4 = {"0,0"};
    	String path = "/Hitori_Spielfelder/Hitori4x4_leicht.csv";
    	InputStream inputStream = getClass().getResourceAsStream(path);
    	ArrayList<String> filteredData = new CSVReader().getSolution(inputStream);
    	assertNotEquals(1, filteredData.size());
    	assertNotEquals(ergebnis4x4[0], filteredData.get(0));
    	assertNotEquals(ergebnis4x4[0], filteredData.get(1));
    	assertNotEquals(ergebnis4x4[0], filteredData.get(2));
    	assertNotEquals(ergebnis4x4[0], filteredData.get(3));
    	assertNotEquals(ergebnis4x4[0], filteredData.get(4));	
    }
    
    @Test
    void getDataTest1() throws FileNotFoundException {
    	String[][] data = {
    			{"3", "3", "1", "4"},
    			{"4", "3", "2", "2"},
    			{"1", "3", "4", "2"},
    			{"3", "4", "3", "2"}};
    	String path = "/Hitori_Spielfelder/Hitori4x4_leicht.csv";
    	InputStream inputStream = getClass().getResourceAsStream(path);
    	assertTrue(Arrays.deepEquals(data, new CSVReader().getData(inputStream, 4)));
    }
    
    @Test
    void getDataTest2() throws FileNotFoundException {
    	String[][] data = {
    			{"3", "0", "1", "4"},
    			{"4", "0", "2", "2"},
    			{"1", "0", "4", "2"},
    			{"3", "0", "3", "2"}};
    	String path = "/Hitori_Spielfelder/Hitori4x4_leicht.csv";
    	InputStream inputStream = getClass().getResourceAsStream(path);
    	assertFalse(Arrays.deepEquals(data, new CSVReader().getData(inputStream, 4)));
    }
    
    @Test
    void readFromFileTest1() {
    	String[] file = { 
    			"3,3,1,4",
    			"4,3,2,2",
    			"1,3,4,2",
    			"3,4,3,2",
    			"",
    			"//Lösung (schwarze Felder)",
    			"1,2",
    			"2,4",
    			"3,2",
    			"4,1",
    			"4,4"};
    	ArrayList<String> fileEntry = new ArrayList<String>();
    	for(int i=0;i<file.length;i++) {
    		fileEntry.add(file[i]);
    	}
    	String path = "/Hitori_Spielfelder/Hitori4x4_leicht.csv";
    	InputStream inputStream = getClass().getResourceAsStream(path);
    	ArrayList<String> ergebnisList = new CSVReader().readFromFile(inputStream);
    	assertEquals(fileEntry, ergebnisList);
    }
    
    @Test
    void readFromFileTest2() {
    	String[] file = { 
    			"3,3,1,0",
    			"4,3,2,0",
    			"1,3,4,0",
    			"3,4,3,0",
    			"",
    			"//Lösung (schwarze Felder)",
    			"1,2",
    			"2,4",
    			"3,2",
    			"4,1",
    			"4,4"};
    	ArrayList<String> fileEntry = new ArrayList<String>();
    	for(int i=0;i<file.length;i++) {
    		fileEntry.add(file[i]);
    	}
    	String path = "/Hitori_Spielfelder/Hitori4x4_leicht.csv";
    	InputStream inputStream = getClass().getResourceAsStream(path);
    	ArrayList<String> ergebnisList = new CSVReader().readFromFile(inputStream);
    	assertNotEquals(fileEntry, ergebnisList);
    }
}
